#ifndef __SYS_SEMA_H__
#define __SYS_SEMA_H__

/**************************************************************************
 *									  *
 * 		 Copyright (C) 1986, Silicon Graphics, Inc.		  *
 *									  *
 *  These coded instructions, statements, and computer programs  contain  *
 *  unpublished  proprietary  information of Silicon Graphics, Inc., and  *
 *  are protected by Federal copyright law.  They  may  not be disclosed  *
 *  to  third  parties  or copied or duplicated in any form, in whole or  *
 *  in part, without the prior written consent of Silicon Graphics, Inc.  *
 *									  *
 **************************************************************************/

#ident	"$Revision: 3.12 $"

/*
 * History mechanism.  History records are linked backwards through the
 * history list.  Thus, a chain of operations can be followed to find
 * out what happened to the semaphore a little easier.  The end of the
 * chain occurs either when the linkage field is zero or the ID of
 * the semaphore recorded does not match that of the chain being followed.
 */
typedef struct semahist_s {
	struct sema_s	*h_sid;		/* semaphore acquired */
	struct proc	*h_proc;	/* acquiring process */
	struct proc	*h_woken;	/* process awoken */
	short		h_pid;		/* its process ID */
	short		h_wpid;		/* is PID */
	char		h_cpuid;	/* processor doing the operation */
	char		h_op;		/* operation type */
	char		h_subop;	/* action performed */
	caddr_t		h_callpc;	/* calling pc */
	struct semahist_s *h_blink;	/* backwards history linkage */
	uint		h_seq;		/* sequence number */
} semahist_t;

extern semahist_t	semahist[];	/* array of history records */
extern unsigned int	histpos;	/* next entry to write */
extern unsigned int	histmax;	/* maximum entries before wrap */
extern lock_t		histlock;	/* lock on update of counter */

#define SEMA_NAMSZ	8
/*
 * the semaphore metering struct - dynamically allocated per semaphore if
 * metering/history is requested
 */
typedef struct k_semameter {
	unsigned int	s_psema;	/* psema's attempted */
	unsigned int	s_vsema;	/* vsema's attempted */
	unsigned int	s_phits;	/* successful psema's */
	unsigned int	s_vnoslp;	/* vsema, nobody waiting */

	unsigned int	s_cpsema;	/* cpsema's attempted */
	unsigned int	s_cphits;	/* successful cpsema's */
	unsigned int	s_cvsema;	/* cvsema attempts */
	unsigned int	s_cvnoslp;	/* cvsema, nobody waiting */

	unsigned int	s_wsema;	/* wsema's attempted */
	unsigned int	s_wnoslp;	/* wsema with nobody there */

	unsigned int	s_unsema;	/* unsema's */
	
	unsigned short	s_nwait;	/* number of waiters */
	unsigned short	s_maxnwait;	/* maximum waiters ever */
	struct sema_s	*m_flink;	/* forward monitor linkage */
	struct sema_s	*m_blink;	/* backward monitor linkage */
	unsigned short	s_pid;		/* last process to acquire */
	unsigned short	s_op;		/* last operation */
	struct proc	*s_proc;	/* last process to acquire */
	semahist_t	*s_blink;	/* for history linkage */
	char		s_name[SEMA_NAMSZ];	/* semaphore name */
} k_semameter_t;


/*
 * Minimal semaphore structure.
 */
typedef struct sema_s {
	/*
	 * Everybody needs some flags!
	 */
	unsigned short	s_flags;	/* semaphore flags */
	/*
	 * Fast access stuff from assembly.
	 */
	short		s_count;	/* current count */
	lock_t		s_lock;		/* lock on count field */
	/*
	 * Slow access stuff - had to block.
	 */
	struct proc	*s_queue;	/* semaphore queue */
	struct proc	*s_qend;	/* tail of queue */

	/*
	 * minimum debugging/metering stuff
	 */
	k_semameter_t	*s_meter;	/* metering struct */
} sema_t;

#define SEMA_NOHIST	01		/* don't collect semaphore history */
#define	semahistoff(s)		(s)->s_flags |= SEMA_NOHIST
#define	semahiston(s)		(s)->s_flags &= ~SEMA_NOHIST

/* semaphore list information */
struct semainfo {
	unsigned int	num;		/* number of semaphore */
	unsigned int	maxnum;		/* maximum number of semaphore */
};
extern sema_t 		*semalist;	/* semaphore list */
extern struct semainfo	semainfo;	/* semaphore info */
extern lock_t		monitorlock;	/* lock on update of semaphore list */

/*
 * Major op code for semaphore log.
 */
# 	define	SEMAOP_PSEMA	1
# 	define	SEMAOP_VSEMA	2
# 	define	SEMAOP_CPSEMA	3
# 	define	SEMAOP_CVSEMA	4
# 	define	SEMAOP_WSEMA	5
#	define	SEMAOP_UNSEMA	6
#	define	SEMAOP_INIT	7
#	define	SEMAOP_FREE	8

/*
 * Minor opcodes for semaphore log.
 */
/* psema */
#	define	SSOP_PHIT	1
#	define	SSOP_PSLP	2
#	define	SSOP_PWAKE	6
#	define	SSOP_PRESIG	7
#	define	SSOP_POSTSIG	8
/* vsema */
#	define	SSOP_VNOWAKE	3
#	define	SSOP_VWAKE	4
/* cpsema */
#	define	SSOP_CPHIT	1
#	define	SSOP_CPMISS	5
/* cvsema */
#	define	SSOP_CVNOWAKE	3
#	define	SSOP_CVWAKE	4
/* wsema */
#	define	SSOP_WMISS	5
#	define	SSOP_WWAKE	4
/* unsema */
#	define	SSOP_RMV	9

#define SMETER(x)		((x)->s_meter)
/*
 * Other constants.
 */
# define TZERO		10

/*
 *	Flag arguments used by semawait.
 */
#define SEMA_WAKE	1
#define SEMA_VSEMA	2
#define SEMA_SPINOP	3

/*
 * Monitor definition
 */
typedef struct monitor {
	sema_t	mon_sema;
	lock_t	mon_lock;		/* lock for accessing queue */
	int	*mon_queue;		/* queue of entries */
	int	(*mon_service)();	/* service procedure */
} mon_t;
/*
 * NOTE - mon_queue points to anything - assumes that first entry
 * is a list pointer
 */

void initmonitor(mon_t *, char *, int (*)());
int pmonitor(mon_t *, int);
int qmonitor(mon_t *, int *);
void vmonitor(mon_t *);

/*
 * Multi-reader locks
 * acccnt == -1 implies someone updating
 */
typedef struct mrlock_s {
	lock_t		mr_acclck;	/* lock on access to mrlock */
	sema_t		mr_wait;	/* wait for access/update lock */
	short		mr_acccnt;	/* count of readers */
	ushort		mr_waitcnt;	/* count of waiting processes */
	struct proc	*mr_last;	/* last process to acquire */
	struct proc	*mr_lastupd;	/* last process to update */
} mrlock_t;
#define MR_ACCESS	0x1
#define MR_UPDATE	0x2
void mrdemote(mrlock_t *);
int mrpromote(mrlock_t *, int);
int ismrlocked(mrlock_t *, int);
void mrunlock(mrlock_t *);
int cmrlock(mrlock_t *, int);
int mrlock(mrlock_t *, int, int);
void mrinit(mrlock_t *, char *);
void mrfree(mrlock_t *);

/*
 * Spinlock Usage
 * Situations:
 * 1) on SP do nothing, MP lock a lock
 * 
 * 2) on SP lock out interrupts, on MP lock out interrupts & lock a lock
 * 
 * 3) on SP and MP lock out interrupts
 *
 * 4) trade a spinlock for a synchronizing semaphore
 *	On SP just grab semaphore.
 * 
 * Routines:
 * 1) splock()/splockmask() - these lock a lock, and return
 *	a magic cookie that must be sent back to spunlock()/spunlockpsema()
 *	On SP systems they are a NOP
 *
 * 2) splockspl() - these raise the spl using the passed function and
 *	on MP systems also lock the lock. spunlockspl() is used to restore the
 *	previous spl and unlock the lock
 *
 * 3) - use spls as before
 *
 * 4) spunlockpsema() - on SP do a psema, on MP unlock lock and do a psema
 *
 *	NOTE: spsemahi/spsema are obsolete. One should no longer assume there
 *		is an implied splhi when locking a lock
 */

/*
 * Function prototypes for semaphore and spinlock calls.
 */
void initlock(lock_t *);
void initnlock(lock_t *, char *);
void spsema(lock_t);
void svsema(lock_t);
void spsemahi(lock_t);
void svsemax(lock_t);
int  splock(lock_t);
int  splockmask(lock_t, int (*)());
void spunlock(lock_t, int);
int  splockspl(lock_t, int (*)());
void spunlockspl(lock_t, int);

void initsema(sema_t *, int);
void initnsema(sema_t *, int, char *);
int semameteron(sema_t *);
void semameteroff(sema_t *);
void freesema(sema_t *);
short valusema(sema_t *);
char *makesname(char *, char *, int);
int psema(sema_t *, int);
int cpsema(sema_t *);
int vsema(sema_t *);
int cvsema(sema_t *);
int vpsema(sema_t *, sema_t *, int);
int svpsema(lock_t, sema_t *, int);
int spunlock_psema(lock_t, int, sema_t *, int);
int spunlockspl_psema(lock_t, int, sema_t *, int);
int appsema(sema_t *, int);
int apvsema(sema_t *);
int apcpsema(sema_t *);
int apcvsema(sema_t *);

/* internal routines used outside sema.c */
int semawait(sema_t *, int, int, int, void (*)(), int, int, int);
#endif /* __SYS_SEMA_H__ */
